'use server';

import Docker from 'dockerode';
import fs from 'fs/promises'; // 使用异步版本的 fs 模块
import path from 'path';
import axios from 'axios';

const docker = new Docker();
const DOCKER_IMAGE_URL = process.env.DOCKER_IMAGE_URL || 'fly6516.synology.me:8080/multilang:latest'; // 使用环境变量

let giteaRepoUrl: string | undefined;
let giteaToken: string | undefined;
let testDataPath: string = 'testdata'; // 默认测试数据目录

/**
 * 设置 Gitea 仓库 URL
 * @param url 仓库 URL
 */
export const setGiteaRepoUrl = async (url: string) => {
    if (!url || !url.startsWith('http')) {
        throw new Error('Invalid repository URL. Please provide a valid URL starting with "http".');
    }
    giteaRepoUrl = url;
};

/**
 * 设置 Gitea Token
 * @param token 仓库 API 访问令牌
 */
export const setGiteaToken = async (token: string) => {
    if (!token || token.trim().length === 0) {
        throw new Error('Invalid token. Please provide a non-empty token.');
    }
    giteaToken = token;
};

/**
 * 设置测试数据在仓库中的路径
 * @param path 测试数据目录路径
 */
export const setTestDataPath = async (path: string) => {
    if (!path || path.trim().length === 0) {
        throw new Error('Invalid test data path. Please provide a non-empty path.');
    }
    testDataPath = path;
};

/**
 * 从 Gitea 仓库中下载指定目录的文件
 * @param dir 目录路径
 * @param tempDir 本地存储的临时目录
 */
const fetchFilesFromGitea = async (dir: string, tempDir: string) => {
    if (!giteaRepoUrl || !giteaToken) {
        throw new Error('Gitea repository URL or token is not set. Please configure them using setGiteaRepoUrl and setGiteaToken.');
    }

    const targetDir = path.join(tempDir, dir);
    await fs.mkdir(targetDir, { recursive: true });

    try {
        const response = await axios.get(`${giteaRepoUrl}/contents/${dir}`, {
            headers: {
                Authorization: `token ${giteaToken}`,
            },
        });

        const files = response.data;
        if (!Array.isArray(files)) {
            throw new Error(`Invalid directory structure for ${dir} in repository.`);
        }

        // 并行下载文件
        await Promise.all(files.filter(file => file.type === 'file').map(async (file) => {
            const fileResponse = await axios.get(file.download_url, {
                headers: {
                    Authorization: `token ${giteaToken}`,
                },
                responseType: 'arraybuffer',
            });

            const filePath = path.join(targetDir, file.name);
            await fs.writeFile(filePath, fileResponse.data);
        }));
    } catch (error) {
        throw new Error(`Failed to fetch files from '${dir}': ${(error as Error).message}`);
    }
};

/**
 * 比较运行结果与正确答案文件
 * @param resultFilePath 运行结果文件路径
 * @param expectedFilePath 正确答案文件路径
 * @returns 是否一致的比较结果
 */
const compareResults = async (resultFilePath: string, expectedFilePath: string): Promise<boolean> => {
    try {
        const result = (await fs.readFile(resultFilePath, 'utf-8')).trim();
        const expected = (await fs.readFile(expectedFilePath, 'utf-8')).trim();
        return result === expected;
    } catch (error) {
        throw new Error(`Failed to read files for comparison: ${(error as Error).message}`);
    }
};

/**
 * 执行多文件代码并测试结果
 * @param params 包含代码文件列表、语言、测试数据文件名和正确答案文件名的参数
 * @returns 运行结果或比较结果
 */
export const runMultiFileCodeWithOptionalTestData = async (params: {
    files: { name: string; content: string }[];
    language: string;
    testDataFiles?: string[];
    expectedAnswerFiles?: string[];
    resultOutputFile?: string;
}) => {
    const { files, language, testDataFiles, expectedAnswerFiles, resultOutputFile } = params;

    if (!files || files.length === 0 || !language) {
        throw new Error('Files and language are required.');
    }

    const supportedLanguages = ['c', 'java', 'python'];
    if (!supportedLanguages.includes(language)) {
        throw new Error(`Language '${language}' is not supported. Only 'c', 'java', and 'python' are supported.`);
    }

    const compileAndRunCmd =
        language === 'c'
            ? 'gcc *.c -o main && ./main'
            : language === 'java'
                ? 'javac *.java && java Main'
                : 'python3 script.py';

    const tempDir = path.join('/tmp', `${Date.now()}`);
    await fs.mkdir(tempDir, { recursive: true });

    try {
        // 写入代码文件
        await Promise.all(files.map(async (file) => {
            const filePath = path.join(tempDir, file.name);
            await fs.writeFile(filePath, file.content);
        }));

        // 检查是否设置了 Gitea 仓库信息
        if (giteaRepoUrl && giteaToken) {
            try {
                await fetchFilesFromGitea(testDataPath, tempDir);
            } catch (error) {
                throw new Error(`Failed to fetch test data from '${testDataPath}': ${(error as Error).message}`);
            }
        } else {
            console.info('No Gitea repository connection set. Proceeding without testdata.');
        }

        // 创建 Docker 容器
        const container = await docker.createContainer({
            Image: DOCKER_IMAGE_URL,
            Cmd: ['sh', '-c', compileAndRunCmd],
            AttachStdout: true,
            AttachStderr: true,
            Tty: false,
            HostConfig: {
                Binds: [`${tempDir}:/workspace`], // 挂载临时目录到容器
            },
            WorkingDir: '/workspace',
        });

        await container.start();

        const logs = await container.logs({ stdout: true, stderr: true, follow: true });

        let output = '';
        logs.on('data', (data: Buffer) => {
            output += data.toString().replace(/[\x00-\x1F\x7F]/g, ''); // 去除不可见字符
        });

        const exitCode = await container.wait();
        await container.remove();

        if (exitCode.StatusCode !== 0) {
            return { error: `Execution failed with exit code ${exitCode.StatusCode}. Output: ${output}` };
        }

        // 如果指定了结果输出文件和正确答案文件，执行比较
        if (resultOutputFile && expectedAnswerFiles && expectedAnswerFiles.length > 0) {
            const resultFilePath = path.join(tempDir, resultOutputFile);

            const comparisonResults = await Promise.all(expectedAnswerFiles.map(async (expectedFile) => {
                const expectedFilePath = path.join(tempDir, testDataPath, expectedFile);
                const isMatch = await compareResults(resultFilePath, expectedFilePath);
                return { expectedFile, isMatch };
            }));

            return { output, comparisonResults };
        }

        return { output };
    } catch (error) {
        return { error: `An error occurred: ${(error as Error).message}` };
    } finally {
        await fs.rm(tempDir, { recursive: true, force: true });
    }
};
