"use client";

import * as monaco from "monaco-editor";
import Editor from "@monaco-editor/react";
import { loader } from "@monaco-editor/react";
import ThemeSelector from "./components/theme-selector";
import FontSizeSlider from "./components/font-size-slider";
import LanguageSelector from "./components/language-selector";
import { useCodeEditorStore } from "@/store/useCodeEditorStore";
import { highlightMonacoEditor } from "@/constants/editor/themes";
import { runMultiFileCode } from "@/actions/multiFileRunner";
import { useState } from "react";

loader.config({ monaco });

export default function MainPage() {
    const { language, theme, fontSize, setEditor } = useCodeEditorStore();
    const [code, setCode] = useState(""); // 存储编辑器代码内容
    const [output, setOutput] = useState(""); // 存储运行结果
    const [loading, setLoading] = useState(false); // 控制加载状态

    const handleRunCode = async () => {
        console.group("Code Execution Debugging");
        console.log("Execution started...");
        console.log("Selected language:", language);
        console.log("Code content:\n", code);

        if (!language || !code.trim()) {
            console.error("Missing language or code content.");
            setOutput("Error: Please select a language and enter code.");
            console.groupEnd();
            return;
        }

        setLoading(true);
        setOutput("Running code...");
        try {
            const files = [
                {
                    name: language === "python" ? "script.py" : language === "java" ? "Main.java" : "main.c",
                    content: code,
                },
            ];
            console.log("Files to execute:", files);

            const result = await runMultiFileCode({ files, language });
            console.log("Execution result:", result);

            if (result.output) {
                console.log("Execution successful. Output:\n", result.output);
                setOutput(result.output);
            } else if (result.error) {
                console.error("Execution error. Details:\n", result.error);
                setOutput(result.error);
            } else {
                console.warn("No output or error returned.");
                setOutput("No output received from execution.");
            }
        } catch (error) {
            console.error("Error during execution:", error);
            setOutput(`Error: ${(error as Error).message}`);
        } finally {
            setLoading(false);
            console.groupEnd();
        }
    };

    return (
        <div className="h-full flex flex-col gap-2">
            <div className="flex items-center px-4 gap-2">
                <LanguageSelector />
                <ThemeSelector />
                <FontSizeSlider />
                <button
                    onClick={handleRunCode}
                    className={`px-4 py-2 rounded ${
                        loading ? "bg-gray-400" : "bg-blue-500 hover:bg-blue-600"
                    } text-white`}
                    disabled={loading}
                >
                    {loading ? "Running..." : "Run Code"}
                </button>
            </div>
            <div className="flex-1">
                <Editor
                    language={language}
                    theme={theme}
                    beforeMount={highlightMonacoEditor}
                    onMount={setEditor}
                    onChange={(value) => {
                        console.log("Editor content updated:", value);
                        setCode(value || "");
                    }}
                    options={{ fontSize, automaticLayout: true }}
                />
            </div>
            <div className="px-4 py-2 bg-gray-900 text-white rounded mt-2 overflow-auto h-48">
                <h3 className="font-bold text-yellow-400">Output:</h3>
                <div className="bg-gray-800 p-2 mt-2 rounded h-32 overflow-y-auto">
                    <pre className="whitespace-pre-wrap">{output || "No output yet."}</pre>
                </div>
            </div>
        </div>
    );
}
